module.exports = (function (state) {
  const R = require('ramda');
  const E = require('simple-encryptor')(state.get('masterPassword'));
  const T = require('egeria-tools');
  const fs = require('fs');
  const path = require('path');

  const frwoptions = 'utf8';
  const prefix = 'ID manager |';
  state.set('prefix', prefix);

  const err = (message, origin, data) => T.err(prefix, message, origin, data);
  const logSilly = (message, origin, data) => T.log(state, 'silly', message, origin, data);

  function fread (fname) {
    let action = 'While reading file ' + fname;
    return new Promise((resolve, reject) => {
      fs.readFile(fname, frwoptions, (e, data) => {
        if (!R.isNil(e)) {
          reject(err(action, e, {fname, frwoptions}));
        }
        else {
          resolve(data);
        }
      });
    });
  }

  function fwrite (fname, data) {
    let action = 'While writing file ' + fname;
    return new Promise((resolve, reject) => {
      fs.writeFile(fname, data, frwoptions, (e, data) => {
        if (!R.isNil(e)) {
          reject(err(action, e, {fname, frwoptions}));
        }
        else {
          resolve(data);
        }
      });
    });
  }

  async function load (identity) {
    logSilly('Loading identity "' + identity + '"');
    if (state.select('identities', identity).exists()) {
      return state.get('identities', identity);
    }
    let action = 'While loading identity ' + identity;
    let directory = state.get('directory');
    let filename = identity + '.encrypted.id';
    let encryptedCredentials;
    try {
      encryptedCredentials = await fread(path.resolve(directory, filename));
    }
    catch(e) {
      throw err(action, e, {directory, filename});
    }
    let decryptedCredentials = E.decrypt(encryptedCredentials);
    state.select('identities', identity).set(decryptedCredentials);
    return decryptedCredentials;
  }

  async function store (identity) {
    logSilly('Saving identity "' + identity.name + '"');
    let action = 'While storing identity ' + identity.name;
    let decryptedCredentials = identity;
    state.select('identities', identity.name).set(decryptedCredentials);
    let directory = state.get('directory');
    let filename = identity.name + '.encrypted.id';
    let encryptedCredentials;
    try {
      encryptedCredentials = E.encrypt(decryptedCredentials);
      await fwrite(path.resolve(directory, filename), encryptedCredentials);
    }
    catch(e) {
      throw err(action, e, {directory, filename, decryptedCredentials, encryptedCredentials}, e);
    }
    return encryptedCredentials;
  }

  return {load, store};
});